
#include "Quaternion.hh"

// Multiplicationen
Quaternion
Quaternion::operator*(const Quaternion& b) const
{
#if 0
  double s[9], t;
  s[0] = (z-y)*(b.y-b.z);
  s[1] = (w+x)*(b.w+b.x);
  s[2] = (w-x)*(b.y+b.z);
  s[3] = (z+y)*(b.w-b.x);
  s[4] = (z-x)*(b.x-b.y);
  s[5] = (z+x)*(b.x+b.y);
  s[6] = (w+y)*(b.w-b.z);
  s[7] = (w-y)*(b.w+b.z);
  s[8] = s[5]+s[6]+s[7];
  t    = .5 * (s[4] +s[8]);

  return Quaternion(s[0]+t-s[5], s[1]+t-s[8], s[2]+t-s[7], s[3]+t-s[6]);
#else
  // Assuming similar speed and multiply-add instructions this version
  // should definitely be faster
  return Quaternion(w*b.w - x*b.x - y*b.y - z*b.z,
		    w*b.x + x*b.w + y*b.z - z*b.y,
		    w*b.y - x*b.z + y*b.w + z*b.x,
		    w*b.z + x*b.y - y*b.x + z*b.w);
#endif
}

void
Quaternion::getRowMatrix(double m[3][3]) const
{
  m[0][0]= 1 - 2*y*y - 2*z*z;
  m[0][1]=     2*x*y - 2*w*z;
  m[0][2]=     2*x*z + 2*w*y;
  m[1][0]=     2*x*y + 2*w*z;
  m[1][1]= 1 - 2*x*x - 2*z*z;
  m[1][2]=     2*y*z - 2*w*x;
  m[2][0]=     2*x*z - 2*w*y;
  m[2][1]=     2*y*z + 2*w*x;
  m[2][2]= 1 - 2*x*x - 2*y*y;
}

void
Quaternion::getColMatrix(double m[3][3]) const
{
  m[0][0]= 1 - 2*y*y - 2*z*z;
  m[1][0]=     2*x*y - 2*w*z;
  m[2][0]=     2*x*z + 2*w*y;
  m[0][1]=     2*x*y + 2*w*z;
  m[1][1]= 1 - 2*x*x - 2*z*z;
  m[2][1]=     2*y*z - 2*w*x;
  m[0][2]=     2*x*z - 2*w*y;
  m[1][2]=     2*y*z + 2*w*x;
  m[2][2]= 1 - 2*x*x - 2*y*y;
}

Quaternion QuatFromRowMatrix(double m[3][3])
{
  double W,X,Y,Z;
  double q0,qx,qy,qz;
  
  W = 1 + m[0][0] + m[1][1] + m[2][2];
  X = 1 + m[0][0] - m[1][1] - m[2][2];
  Y = 1 - m[0][0] + m[1][1] - m[2][2];
  Z = 1 - m[0][0] - m[1][1] + m[2][2];


  if (W>=X && W>=Y && W>=Z) { // W the biggest
    q0 = sqrt(W) / 2;
    qx = (m[2][1] - m[1][2]) / ( 4 * q0 );
    qy = (m[0][2] - m[2][0]) / ( 4 * q0 );
    qz = (m[1][0] - m[0][1]) / ( 4 * q0 );

  } else if (X>=W && X>=Y && X>=Z) { // X the biggest
    qx = sqrt(X) / 2;
    q0 = (m[2][1] - m[1][2]) / ( 4 * qx );
    qy = (m[1][0] + m[0][1]) / ( 4 * qx );
    qz = (m[0][2] + m[2][0]) / ( 4 * qx );

  } else if (Y>=W && Y>=X && Y>=Z) { // Y the biggest
    qy = sqrt(Y) / 2;
    q0 = (m[0][2] - m[2][0]) / ( 4 * qy );
    qx = (m[1][0] + m[0][1]) / ( 4 * qy );
    qz = (m[2][1] + m[1][2]) / ( 4 * qy );

  } else { // Z the biggest
    qz = sqrt(Z) / 2;
    q0 = (m[1][0] - m[0][1]) / ( 4 * qz );
    qx = (m[0][2] + m[2][0]) / ( 4 * qz );
    qy = (m[2][1] + m[1][2]) / ( 4 * qz );

  }
  return(Quaternion(q0 ,qx, qy, qz));
 
}
