#ifndef QUATERNION_HH
#define QUATERNION_HH

#include <math.h>

class Quaternion
{ 
public:
  // Construktors
  Quaternion()		{ w=1; x= y= z= 0; }
  Quaternion(double v0, double v1, double v2, double v3)
			{ w= v0; x= v1; y= v2; z= v3; }
  Quaternion(double angle, double v[3])
			{ w= angle; x= v[0]; y=v[1]; z=v[2]; }

  Quaternion(const Quaternion& q)
			{ w= q.w; x= q.x; y= q.y; z= q.z; }
 

  // Assignement
  const Quaternion& operator=(const Quaternion& q)
			{ w= q.w; x= q.x; y= q.y; z= q.z; return *this; }

  // Addition
  Quaternion operator+(const Quaternion& b) const
			{ return Quaternion(w+b.w, x+b.x, y+b.y, z+b.z); } 

  Quaternion operator-(const Quaternion& b) const
			{ return Quaternion(w+b.w, x+b.x, y+b.y, z+b.z); } 
  Quaternion operator-() const
			{ return Quaternion(-w, -x, -y, -z); } 
   
  // Multiplication
  Quaternion operator*(const Quaternion& q) const;
  Quaternion operator*=(const Quaternion& q) { *this= *this*q; return *this; }
  Quaternion operator*(double a) const
			{ return Quaternion(w*a, x*a, y*a, z*a); }
  Quaternion operator/(const Quaternion& q) const
			{ return (*this) * q.inv(); }
  Quaternion operator/(double a) const
			{ return Quaternion(w/a, x/a, y/a, z/a); }

  inline friend Quaternion operator*(double a, const Quaternion& q);

  // Conjugation
  Quaternion conj() const
			{ return Quaternion(w, -x, -y, -z); } 
 
  // Scalar product
  double operator|(const Quaternion& q) const
			{ return w*q.w + x*q.x + y*q.y + z*q.z; }

  /// Norm/normalization of Quaternionernion
  double norm() const	{ return sqrt(*this|*this); }

  const Quaternion& normalize()
			{ return ((*this)= (*this) / this->norm()); }

  Quaternion inv() const
			{ return this->conj().normalize(); }
 
  void getRowMatrix(double [3][3]) const;
  void getColMatrix(double [3][3]) const;

  double operator[](int i) const
    { switch (i)
      { case 0: return w; case 1: return x;
	case 2: return y; case 3: return z;
	default: return 0;
      }
    }

private:
  double w, x, y, z;
};

inline Quaternion
operator*(double a, const Quaternion& q)
{
  return Quaternion(a*q.w, a*q.x, a*q.y, a*q.z);
}

Quaternion QuatFromRowMatrix(double rotm[3][3]);

#endif
