function intrpar=match2intr(outname,indir,varargin)
%intr=match2intr(outname,indir,matchfile1,matchfile2,...)
%e.g. intr=match2intr('img12','B','Xcolor1','Xcolor2')
%calibrates from data files B/Xcolor1.match, B/Xcolor2.match, etc
% and outputs the intrinsics  and the extrinsic of the FIRST data
% file in img12.fin

num=nargin-2;

if (num<1)
    fprintf(1,'\nUsage: match2fin <outputFile(.fin)> <directory> <im1.match> [im2.match] ...\n');
    fprintf(1,'  e.g. match2fin foo . image1 image2\n\n');
    exit
end


vlist=cell(1,num);
for i=1:num
  fname=[];
  
  tmpname=strcat(varargin{i});
  if (exist(tmpname)==2)
      fname=tmpname;
      fprintf('Found file %s\n',fname);
  else
      fprintf('Did not find file %s\n',tmpname);
  end

 tmpname=strcat(varargin{i},'.match');
  if (exist(tmpname)==2)
      fname=tmpname;
       fprintf('Found file %s\n',fname);
  else
      fprintf('Did not find file %s\n',tmpname);
  end

  tmpname=strcat(indir,'\',varargin{i},'.match');
  if (exist(tmpname)==2)
      fname=tmpname;
      fprintf('Found file %s\n',fname);
  else
      fprintf('Did not find file %s\n',tmpname);
  end

  tmpname=strcat(indir,'/',varargin{i},'.match');
  if (exist(tmpname)==2)
      fname=tmpname;
       fprintf('Found file %s\n',fname);
  else
      fprintf('Did not find file %s\n',tmpname);
  end
  

  if (~isempty(fname))
    %immat=load(fname,'-ascii');
    fid=fopen(fname,'r');
    immat=fscanf(fid,'%f %f %f %f %f %f %f %f\n');
    fclose(fid);
    immat=reshape(immat,8,[]);
    immat=immat';
    size(immat);
    fprintf(1,'\n%s loaded.\n',fname); 
    vlist{i}=immat;
  else
    fprintf(1,'Could not load file %s\n',varargin{i});
    exit
  end
  
end

cameratype='generic';
[par,pos,iter,res,er,C]=feval('circal',cameratype,vlist{:});
sys=configc(cameratype);
a=invmodel(cameratype,par);
intrpar=[sys(1:4);par;a];
fullpar=[intrpar; pos(:,1)];

fname=strcat(outname,'.fin');
infofname=strcat(outname,'.info');
fid=fopen(infofname,'w');
finfid=fopen(fname,'w');
fprintf(finfid,'%f\n',fullpar);
fclose(finfid);
%save(fname,'fullpar','-ascii');
fprintf(1,'\nIntrinsic and Extrinsic paramters saved in %s.\n',fname);

% report errors we need:
n=size(er,1)/2;
er2=reshape(er,n,2);
sq_error = er2(:,1).*er2(:,1) + er2(:,2).*er2(:,2);
abs_error = sqrt(sq_error);
mean_error = mean(abs_error);
std_error = std(abs_error);
fprintf(1,'\nMean absolute error: %f [pix]\n',mean_error);
fprintf(fid,'\nMean absolute error: %f [pix]\n',mean_error);
fprintf(1,'STD of absolute error: %f [pix]\n',std_error);
fprintf(fid,'STD of absolute error: %f [pix]\n',std_error);
[m, i]=max(abs_error);
fprintf(1,'Max absolute error: %f [pix], occuring at index %f.\n',m,i);
fprintf(fid,'Max absolute error: %f [pix], occuring at index %f.\n',m,i);

calib.intrpar = intrpar;
calib.mean_error=mean_error;
calib.std_error = std_error;
calib.max_error = m;
calib.max_idx = i;
fname=strcat(outname,'.mat');
save(fname,'calib')
fprintf(1,'\nCamera parameters and errors saved in %s. \n',fname);

fclose(fid);



