function fullcalib=xyzuv2fin(dirname,xyzuvname)
% function fullcalib=xyzuv2fin(indirname,xyzuvname)
% estimates both the intrinsic and extrinsic parameters from a .xyzuv
% file and outputs a .fin file in the given directory
% e.g. xyzuv2fin('A','movie2') reads A/movie2.xyzuv and outputs
% movie2.fullcalib.fin
% It returns a struct that contains mean,std,max error (and corresponding index)
% and the same struct is saved in movie2.fullcalib.mat
cameratype='generic';
inname=strcat(dirname,'/',xyzuvname,'.xyzuv')
feval('load',inname,'-ascii');
fprintf('\n%s loaded.',inname);
[par,pos,iter,res,er,C]=feval('cacal',cameratype,eval(xyzuvname));
sys=configc(cameratype);
a=invmodel(cameratype,par);
fullpar=[sys(1:4);par;a;pos];

fname=strcat(xyzuvname,'.fullcalib.fin');
feval('save',fname,'fullpar','-ascii')
fprintf(1,'\nIntrinsic and extrinsic paramters save in %s.\n',fname);

% report errors we need:
n=size(er,1)/2;
er2=reshape(er,n,2);
sq_error = er2(:,1).*er2(:,1) + er2(:,2).*er2(:,2);
abs_error = sqrt(sq_error);
mean_error = mean(abs_error);
std_error = std(abs_error);
fprintf(1,'\nMean absolute error: %f [pix]\n',mean_error);
fprintf(1,'STD of absolute error: %f [pix]\n',std_error);
[m, i]=max(abs_error);
fprintf(1,'Max absolute error: %f [pix], occuring at index %f.\n',m,i);

fullcalib.fullpar = fullpar;
fullcalib.mean_error=mean_error;
fullcalib.std_error = std_error;
fullcalib.max_error = m;
fullcalib.max_idx = i;
fname=strcat(xyzuvname,'.fullcalib.mat');
feval('save',fname,'fullcalib')
fprintf(1,'\nCamera parameters and errors saved in %s. \n',fname);
