function extrcalib=xyzuv_intr2fin(indirname,xyzuvname,intrname)
%
% function extrcalib=xyzuv_intr2fin(indirname,xyzuvname,intrname)
% estimates the extrinsic parameters from a .xyzuv file and a .fin file.
% The .fin file providess the intrinsic paramters. and
% the .xyzuv file provides 3D-2D correspondances
% The intrinsic and extrinsic parameters are written in a .fin
% in the directory specified.
%
% e.g. ext2fin('A','movie2','im135') reads A/movie2.xyzuv and A/im135.fin
% and outputs the full camera parameter in movie2_im135.fin.
% It returns a struct that contains the extrinsic parameters, the
% mean, std, max error and corresponding index, and saves those
% in file movie2_im135.mat

fname=strcat(indirname,'/',xyzuvname,'.xyzuv')
feval('load',fname,'-ascii')
fprintf('\n%s loaded.\n',fname);

%fname=strcat(indirname,'/',intrname,'.fin');
fname=strcat(indirname,'/',intrname,'.fin');
feval('load',fname,'-ascii')
fprintf('\n%s loaded.\n',fname);

cameratype='generic';
intrinsic=eval(intrname);
intrinsic=intrinsic(1:20);% cut off extrinsics
par=intrinsic(5:12);
[pos,iter,res,er,C]=feval('extcal',cameratype,eval(xyzuvname),par);

fullpar=[intrinsic;pos];

fname=strcat(xyzuvname,'_',intrname,'.fin');
feval('save',fname,'fullpar','-ascii')
fprintf(1,'\nIntrinsic and extrinsic paramters saved in %s.\n',fname);

% report errors we need:
n=size(er,1)/2;
er2=reshape(er,n,2);
sq_error = er2(:,1).*er2(:,1) + er2(:,2).*er2(:,2);
abs_error = sqrt(sq_error);
mean_error = mean(abs_error);
std_error = std(abs_error);
fprintf(1,'\nMean absolute error: %f [pix]\n',mean_error);
fprintf(1,'STD of absolute error: %f [pix]\n',std_error);
[m, i]=max(abs_error);
fprintf(1,'Max absolute error: %f [pix], occuring at index %f.\n',m,i);

extrcalib.fullpar = fullpar;
extrcalib.mean_error=mean_error;
extrcalib.std_error = std_error;
extrcalib.max_error = m;
extrcalib.max_idx = i;

% save result in a .mat file
fname=strcat(xyzuvname,'_',intrname,'.mat');
feval('save',fname,'extrcalib')
fprintf(1,'\nCamera parameters and errors saved in %s. \n',fname);
