#include <iostream.h>
#include <assert.h>

/*
class Vector {
private:
  double *data;
  int r;
public:
  Vector(int i) : r(i) { data = new double[r]; }
  ~Vector(void)        { delete[] data; }

  double &operator[](int i)
    { return data[i]; }

  Vector &operator =(const Vector &v)
    { assert(r=v.r); for (int i=0; i<r; i++) data[i]=v.data[i]; }
};
*/

class Matrix {
private:
  double *_data;
  double **data;
  int   r,c;

  void create(void)
    {
      _data = new double[r*c];
      data = new double*[r];
      for (int k=0; k<r; k++) data[k] = &_data[k*c];
    }

public:
  Matrix(int i, int j)
    : r(i), c(j)
    { create(); }

  Matrix(const Matrix &m)
    : r(m.r), c(m.c)
    { create(); for (int k=0; k<r*c; k++) _data[k] = m._data[k]; }

  ~Matrix()
    { delete[] _data; delete[] data; }

  Matrix &operator =(const Matrix &m)
    {
      assert(r == m.r && c == m.c);
      for (int i=0; i<r*c; i++) _data[i] = m._data[i];
      return *this;
    }

  Matrix &set(double *a) 
    {
      for (int i=0; i<c*r; i++) _data[i] = a[i];
      return *this;
    }

  Matrix &set(float *a) 
    {
      for (int i=0; i<c*r; i++) _data[i] = a[i];
      return *this;
    }


  Matrix &identity(void);

  Matrix &ones(void);
  Matrix &zero(void);

  Matrix &invert(void);

  double & operator()(int i, int j);
    // set and get values
  Matrix transpose(void);
    // return the transposed matrix (new, by copy)
  Matrix &operator+=(const Matrix &a);
    // return self
  friend Matrix operator*(const Matrix &a, const Matrix &b);
    // return the resulting matrix (new, by copy)
  friend void mult(const Matrix &a, double *b, double *c);
    // c = a * b
  friend Matrix Tmult(const Matrix &a, const Matrix &b);
    // return At * B

  friend void Tmult(const Matrix &a, double *b, double *c);
    // c = At * b
  friend void square(const Matrix &a, Matrix &ata);
    // ata = At * A

  friend ostream &operator <<(ostream &os, const Matrix &m);

  friend int gauss_jordan(Matrix &a, Matrix &b);
    // solve Ax = B, replace A with its invertion, B with
    // solutions. From numerical recipes
    // return 0 if matrix A singular
  friend int gauss_jordan_fast(Matrix &a, double *b);
    // solve Ax = b, A is scrambled afterwards
  void HouseholderQR(void);
    // (from Golub and Van Loan)
    // finds Householder matrices H1,...,Hn s.t. if 
    // Q = H1 ... Hn, then QtA = R is upper triangular
    // the matrix is overwritten by the upper triangular
    // part of R, the lower part contains parts of the 
    // Householder vectors
  void HouseholderLSQ(double *b);
    // solve Ax = b using Householder transformation
    // and QR decomposition (from Golub and Van Loan)
    // the matrix will be messed up in the end
  double maxEig(void);
    // return the maximum eigenvalue, use the power method
    // really, the maximum absolute eigenvalue
  double minEig(void);
    // return the minimum eigenvalue, use inverse iteration
    // doesn't work if min eigenvalue < 0
};


class LSQ {
private:
  int ac, bc;
  Matrix *AtA;       // (acXac)
  Matrix *AtB;       // (acXbc)

public:
  LSQ(int i, int j);
    // solve A x = B as x = (AtA)^-1 At B
    // A is (nXi), x is (iXj), b is (nXj) matrix

  ~LSQ(void);

  void clear(void);

  void add(const double *A, const double *B);
  void add(const float *A, const float *B);
    // say each row of A is x y z and row of B is X Y
    // denote sum as S() then AtA and AtB become
    //
    // |S(xx) S(xy) S(xz)|     |S(xX) S(xY)|
    // |S(xy) S(yy) S(yz)| and |S(yX) S(yY)|
    // |S(xz) S(yz) S(zz)|     |S(zX) S(zY)|
    //

  Matrix &solve(void);

  double operator()(int i, int j)
    { return (*AtB)(i,j); }
};
